//===================================================================================================================================
// Transmission Scans of Rayleigh Particles - Markus Selmke (markus.selmke@gmx.de, selmke@rz.uni-leipzig.de)
//===================================================================================================================================

import blobDetection.*;
import controlP5.*;
ControlP5 controlP5;

public int CSizeX=450, CSizeZ=850, Nx=21, Nz=31, Ntheta=10, Ns=11; //CANVAS SIZE, grid lines, fosuc spot size, active vertex spot size
public float x0Max=4, z0Max=3, thetaMin=0.01, dtheta=0.2, sMin=0.05, ds=0.025;

public float[][][] AxIm, AxRe; //array storing the data. Last index is the data-set index.
public PFont font, CP5font;
public boolean ContoursToggle=false, AxisToggle=true, interpToggle=false, BeamToggle=true, InvertColorToggle=false, GridToggle=true, ToggleDynamic=false; //Toggles for Dynamic mode, Image Display
public float AlphaRe=1.327, AlphaIm=-0.098, w0=280, kR=0.433, w0Max=1000, r, sigmaInc, thetamaxV, thetamaxBythetaDivV, sV; //-0.067
public float mouseV, maxRe, maxIm, maxV, minV, maxAbsV, AlphaColor=1, w0Scale, z0Scale, nRe=0.175, nIm=-3.46, ProfileHeight=90;
public int CSs = int(CSizeZ/3.6), CSx = 20, CSw=20, is=0, iis, itheta=0, iitheta , ki, kii, BL, LineProfile = 0;
PImage im_thetadivbythetamax, im_thetamax, im_thetadiv, im_alphaIm, im_alphaRe, im_w0, im_nr, im_ni, im_kR, im_s;
PImage im_mw0, im_mzR, im_zR, im_alphak3Re, im_alphak3Im, im_Pd, im_zbyzR, im_xbyw0;

float levels = 5;  // number of contours
BlobDetection[] theBlobDetection = new BlobDetection[int(levels)]; // Array of BlobDetection Instances

int bh=20, bw=120, i, j, xi, yi;
float x,y,ii,ji, xc, yc, rC, gC, bC, aC, ri;
Table Ax;
PImage TransmissionIm=createImage(CSizeX,CSizeZ,ARGB); //interactive background image
PImage TransmissionImG=createImage(CSizeX,CSizeZ,ALPHA); //needed for high-res contours
PImage TransmissionImGlr=createImage(2*Nx-1,2*Nz-1,ALPHA); //needed for real-time contours

//Start-Up Routine, runs before continiously excecuted draw-routine
void setup(){
 size(CSizeX,CSizeZ); //canvas size
 smooth();
 frameRate(27);
 
 //load data
  Ax = loadTable("Ax_Allfine.dat", "tsv");
  
 //load images (latex formulas for captions) 
  im_thetadivbythetamax = loadImage("formula_thetadivbythetamax.png");
  im_thetamax = loadImage("formula_thetamax.png");
  im_thetadiv = loadImage("formula_thetadiv.png");
  im_alphaIm = loadImage("formula_alphaIm.png");
  im_alphak3Im = loadImage("formula_alphak3Im.png");
  im_alphaRe = loadImage("formula_alphaRe.png");
  im_alphak3Re = loadImage("formula_alphak3Re.png");
  im_nr = loadImage("formula_nr.png");
  im_ni = loadImage("formula_ni.png");
  im_kR = loadImage("formula_kR.png");
  im_s = loadImage("formula_s.png");
  im_w0 = loadImage("formula_w0.png");
  im_mw0 = loadImage("formula_mw0.png");
  im_zR = loadImage("formula_zR.png");
  im_mzR = loadImage("formula_mzR.png");
  im_Pd = loadImage("formula_dPbyPinc.png");
  im_zbyzR = loadImage("formula_zbyzR.png");
  im_xbyw0 = loadImage("formula_xbyw0.png");
  
 //Initialize arrays storing Real and Imaginary Parts of Ax
  AxIm = new float[Nx][Nz][Ns*Ntheta]; //2D-array of size (Nx x Nz x ParameterCombinations), stores x-coordinates of vertices
  AxRe = new float[Nx][Nz][Ns*Ntheta]; //2D-array of size (Nx x Nz x ParameterCombinations), stores x-coordinates of vertices 
  is=0; itheta=0; //data-set indexes for first load
  
 //Setup the GUI
  font = createFont("Arial", 12); 
  textFont(font);
  
  controlP5 = new ControlP5(this);
    ControlFont CP5font = new ControlFont(font); 
    controlP5.setControlFont(CP5font);
  
  Slider AlphaReSlider = controlP5.addSlider("Re_alpha", 0, 3, 0, 79, 10, bw, bh);
      AlphaReSlider.setValue(AlphaRe);
  Slider AlphaImSlider = controlP5.addSlider("Im_alpha", -3, 0, 0, 79, 40, bw, bh);
      AlphaImSlider.setValue(AlphaIm);
  Slider sSlider = controlP5.addSlider("sSlider", sMin, sMin+(Ns-1)*ds, 0, 89, CSizeZ - 30, bw, bh);
      sSlider.setValue(sMin+is*ds);
  Slider thetamaxSlider = controlP5.addSlider("thetamaxSlider", thetaMin, thetaMin+(Ntheta-1)*dtheta, 0, 89, CSizeZ - 60, bw, bh);
      thetamaxSlider.setValue(thetaMin+itheta*dtheta);
//  Slider kRSlider = controlP5.addSlider("kRSlider", 0, 1, 0, CSizeX - 60, 80, 50, bh);
//      kRSlider.setValue(kR);
  
  Button ComputeContoursButton = controlP5.addButton("ComputeContours",1.0,10,70,20,20);
  Toggle Interpolate = controlP5.addToggle("Interpolate", interpToggle,10,95,20,20);
      Interpolate.setValue(true);
  Toggle InvertColor = controlP5.addToggle("InvertColor", InvertColorToggle,10,120,20,20);
      InvertColor.setValue(true);
  //Toggle Grid = controlP5.addToggle("Grid", GridToggle,10,145,20,20);
      //Grid.setValue(true);
  
  AlphaReSlider.captionLabel().toUpperCase(false);
  AlphaImSlider.captionLabel().toUpperCase(false);
  Interpolate.captionLabel().toUpperCase(false);
  InvertColor.captionLabel().toUpperCase(false);
  sSlider.captionLabel().toUpperCase(false);
  thetamaxSlider.captionLabel().toUpperCase(false);
  ComputeContoursButton.captionLabel().toUpperCase(false);
  //Grid.captionLabel().toUpperCase(false);
  AlphaReSlider.setCaptionLabel("");
  AlphaImSlider.setCaptionLabel("");
  sSlider.setCaptionLabel("");
  thetamaxSlider.setCaptionLabel("");
  Interpolate.setCaptionLabel("");
  ComputeContoursButton.setCaptionLabel("");
  InvertColor.setCaptionLabel("");
  //Grid.setCaptionLabel("");
 
 //Put the read-in data into the arrays 
  for (i=0; i<Nx; i=i+1) {
    for (j=0; j<Nz; j=j+1) {
       for (iis=0; iis<Ns; iis++) { //iis 0..2
          for (iitheta=0; iitheta<Ntheta; iitheta++) { //iitheta 0..1
            kii = Ntheta*iis + iitheta;
            AxRe[i][Nz-j-1][kii]=Ax.getFloat(j*Nx + i, 2 + 2*kii); //Real part
            AxIm[i][Nz-j-1][kii]=Ax.getFloat(j*Nx + i, 3 + 2*kii); //Im part
         }
       }
     }
  }
}

void Grid(boolean theFlag) {
  GridToggle=theFlag;
}

void Interpolate(boolean theFlag) {
  interpToggle=!theFlag;
}

void InvertColor(boolean theFlag) {
  InvertColorToggle= theFlag;
  BL = (BL+1) % 2;
}

void Re_alpha(float theValue) {
  AlphaRe=theValue;
  ContoursToggle=false;
}

void Im_alpha(float theValue) {
  AlphaIm=theValue;
  ContoursToggle=false;
}

void sSlider(float theValue) {
  is=floor((theValue-sMin)/ds);
  ContoursToggle=false;
}

void thetamaxSlider(float theValue) {
  itheta=floor((theValue-thetaMin)/dtheta);
  ContoursToggle=false;
}

void ComputeContours(int theValue) {
  GetContours();
  ContoursToggle=true;
}


void draw(){
  //setup canvas and set colorspace to be RGB with values between 0 and 1
    frame.setTitle("Transmission");
    colorMode(RGB,1,1,1,1);
    ellipseMode(CENTER); //draws cirles, uses center coordinates
    rectMode(CORNER);
    tint(1,1);
  
  //=======UPDATE Indices==================================================================
    ki = Ntheta*is + itheta; //current data-set index
    sV = sMin+ds*is; //current focusing value s=w0/2zR
    thetamaxBythetaDivV = thetaMin+dtheta*itheta; //current collection angle ratio
    thetamaxV = thetamaxBythetaDivV*(2*sV); //current collection angle (absolute)
    sigmaInc = 2*PI*(1-exp(-2*pow(tan(thetamaxV),2)/pow(2*sV,2)))/(2*sV); //incident cross-section (approx., not using BSCs)
    
  //=======UPDATE IMAGE=====================================================================
    background(0);
    UpdateBG();
    if (interpToggle==true) {
      GetContours(); //live drawing of contours only when interpolation is OFF!
      ContoursToggle=true;
    }
    image(TransmissionIm,0,0); 
    
  //=======Draw Beam-waists=================================================================
  if (BeamToggle==true) {  
    noFill(); stroke(1,0,0,0.2); strokeWeight(2); //fill(0,0,255,50);
    beginShape();
    for (y=-CSizeZ-2; y<CSizeZ+2; y=y+1) {
      vertex(CSizeX/2+((CSizeX/2)/x0Max)*sqrt(1+pow(2*(y-CSizeZ/2)*z0Max/(CSizeZ/2),2)),y);
    }
    endShape();
    beginShape();
    for (y=-CSizeZ-2; y<CSizeZ+2; y=y+1) {
      vertex(CSizeX/2-((CSizeX/2)/x0Max)*sqrt(1+pow(2*(y-CSizeZ/2)*z0Max/(CSizeZ/2),2)),y);
    }
    endShape();
  }
    
  //=======Draw CONTOURS (No computation)===================================================  
    if (ContoursToggle==true) {
      for (int i=0; i<levels; i++) {
        drawContours(i);
      }
    }  
    
  //=======LINE PROFILE====================================================================
    strokeWeight(1);
    if (LineProfile==1) {
      stroke(1-BL,1-BL,1-BL,0.5); //profile-curve
      fill(1,1,1,0.1); //box
      rect(CSizeX/2 + 10,5,CSizeX/2-20,2*ProfileHeight);
      stroke(1-BL,1-BL,1-BL,1); //zero-line
      line(CSizeX/2 + 10,ProfileHeight+5,CSizeX-10,ProfileHeight+5);
      line(3*CSizeX/4,30,3*CSizeX/4,5+2*ProfileHeight);
      stroke(1-BL,1-BL,1-BL,0.5); //profile-curve
      for (xi=0; xi<CSizeX; xi++) {
        ri = r;
        r = signum(mouseY-CSizeZ/2)*AlphaRe*Interp2D(AxIm, abs(xi-CSizeX/2)*w0Scale, abs(mouseY-CSizeZ/2)*z0Scale);
        r = r + AlphaIm*Interp2D(AxRe, abs(xi-CSizeX/2)*w0Scale, abs(mouseY-CSizeZ/2)*z0Scale);
        r = -ProfileHeight*(r/sigmaInc)/maxAbsV;
        if (xi>0) {
          line(CSizeX/2 + 10 + (CSizeX/2-20)*xi/CSizeX, ProfileHeight+5+ri, CSizeX/2 + 10 + (CSizeX/2-20)*(xi+1)/CSizeX, ProfileHeight+5+r);
        }
      }
      stroke(1-BL,1-BL,1-BL,0.5);
      line(0,mouseY,CSizeX,mouseY);
      fill(0.5,0.5,0.5,1);
      text("x",CSizeX-20,ProfileHeight+20);
    } else if (LineProfile==2) {
      stroke(1-BL,1-BL,1-BL,0.5); //profile-curve
      fill(1,1,1,0.1); //box
      rect(CSizeX/2 + 10,5,CSizeX/2-20,2*ProfileHeight);
      stroke(1-BL,1-BL,1-BL,1); //zero-line
      line(CSizeX/2 + 10,ProfileHeight+5,CSizeX-10,ProfileHeight+5);
      line(3*CSizeX/4,30,3*CSizeX/4,5+2*ProfileHeight);
      stroke(1-BL,1-BL,1-BL,0.5); //profile-curve
      for (yi=0; yi<CSizeZ; yi++) {
        ri = r;
        r = signum(yi-CSizeZ/2)*AlphaRe*Interp2D(AxIm, abs(mouseX-CSizeX/2)*w0Scale, abs(yi-CSizeZ/2)*z0Scale);
        r = r + AlphaIm*Interp2D(AxRe, abs(mouseX-CSizeX/2)*w0Scale, abs(yi-CSizeZ/2)*z0Scale);
        r = -ProfileHeight*(r/sigmaInc)/maxAbsV;
        if (yi>0) {
          line(CSizeX/2 + 10 + (CSizeX/2-20)*yi/CSizeZ, ProfileHeight+5+ri, CSizeX/2 + 10 + (CSizeX/2-20)*(yi+1)/CSizeZ, ProfileHeight+5+r);
        }
      }
      stroke(1-BL,1-BL,1-BL,0.5); 
      line(mouseX,0,mouseX,CSizeZ);
      fill(0.5,0.5,0.5,1);
      text("z",CSizeX-20,ProfileHeight+20);
    }
    stroke(1-BL,1-BL,1-BL,1); strokeWeight(1); fill(1-BL,1-BL,1-BL,1);
    ellipse(mouseX,mouseY,2,2);
    
  //=======DRAW COLOR SCALE=================================================================
    for (yi=-int(CSs*abs(minV)/maxAbsV); yi<CSs*maxV/maxAbsV; yi=yi+1) {
      color ColorV = ColorF(-1.0*yi/CSs); 
      rC=red(ColorV); gC=green(ColorV); bC=blue(ColorV); aC=alpha(ColorV);
      stroke(rC,gC,bC,aC);
      line(CSx,CSizeZ/2-yi,CSx+CSw,CSizeZ/2-yi);
    }
    //outline colorscale
    stroke(1-BL,1-BL,1-BL,1); strokeWeight(1); fill(1-BL,1-BL,1-BL,1);
    line(CSx,CSizeZ/2+CSs*abs(minV)/maxAbsV,CSx,CSizeZ/2-CSs*maxV/maxAbsV);
    line(CSx+CSw,CSizeZ/2+CSs*abs(minV)/maxAbsV,CSx+CSw,CSizeZ/2-CSs*maxV/maxAbsV);
    line(CSx,CSizeZ/2+CSs*abs(minV)/maxAbsV,CSx+CSw,CSizeZ/2+CSs*abs(minV)/maxAbsV);
    line(CSx,CSizeZ/2-CSs*maxV/maxAbsV,CSx+CSw,CSizeZ/2-CSs*maxV/maxAbsV);
    text(str(maxV), CSx+CSw-25, CSizeZ/2-CSs*maxV/maxAbsV-10);
    text("0", CSx+CSw/2-3, CSizeZ/2+4);
    text(str(minV), CSx+CSw-25, CSizeZ/2+CSs*abs(minV)/maxAbsV+15);
  
  //========DRAW AXIS======================================================================
  if (AxisToggle==true) {
    tint(1-BL,1); fill(1-BL,1-BL,1-BL,1);
      image(im_Pd,CSx,CSizeZ/2+CSs*abs(minV)/maxAbsV+27);
      image(im_mzR, CSizeX/2-41, CSizeZ/2 + (CSizeZ/2)*(0.5/z0Max) - 8);
      image(im_zR, CSizeX/2-28, CSizeZ/2 - (CSizeZ/2)*(0.5/z0Max)-8);
      image(im_w0, CSizeX/2 + (CSizeX/2)/x0Max-7, CSizeZ/2+8);
      image(im_mw0, CSizeX/2 - (CSizeX/2)/x0Max-7, CSizeZ/2+8);
    strokeWeight(1); strokeJoin(ROUND); stroke(1-BL,1-BL,1-BL,0.5); fill(1-BL);
    ellipse(CSizeX/2,CSizeZ/2,5,5); //Coordinate center
    line(0,CSizeZ/2,CSizeX,CSizeZ/2); //x-axis
    line(CSizeX/2,0,CSizeX/2,CSizeZ); //z-axis
    //--x- and z-markers
    for (xi=-int(x0Max); xi<=x0Max; xi++) {
      stroke(1-BL,1-BL,1-BL,0.5);
        line(CSizeX/2 + xi*(CSizeX/2)/x0Max,CSizeZ/2+5,CSizeX/2 + xi*(CSizeX/2)/x0Max,CSizeZ/2-5);
      if (GridToggle==true) {
        stroke(1-BL,1-BL,1-BL,0.1);
        line(CSizeX/2 + xi*(CSizeX/2)/x0Max,0,CSizeX/2 + xi*(CSizeX/2)/x0Max,CSizeZ);
      }
    }
    for (yi=-2*int(z0Max); yi<=2*z0Max; yi++) {
      stroke(1-BL,1-BL,1-BL,0.5);
      line(CSizeX/2-5,CSizeZ/2 + yi*(CSizeZ/2)*(0.5/z0Max),CSizeX/2+5,CSizeZ/2 + yi*(CSizeZ/2)*(0.5/z0Max));
      if (GridToggle==true) {
        stroke(1-BL,1-BL,1-BL,0.1);
        line(0,CSizeZ/2 + yi*(CSizeZ/2)*(0.5/z0Max),CSizeX,CSizeZ/2 + yi*(CSizeZ/2)*(0.5/z0Max));
      }
    }
  }
  
  
  //=======DRAW ANNOTATIONS FOR SLIDERS AND BUTTONS========================================
  tint(1-BL,1); fill(1-BL,1-BL,1-BL,1);
  
  image(im_Pd,CSx,CSizeZ/2+CSs*abs(minV)/maxAbsV+27);
  image(im_alphak3Re, 10, 10-2);
  image(im_alphak3Im, 10, 40-2);
  image(im_thetadivbythetamax, 10, CSizeZ-60);
    text(nf(thetamaxBythetaDivV,1,3), 215, CSizeZ-45); //"itheta: "+ str(itheta)+ ", thetaV: "+
  image(im_s, 10, CSizeZ-30);
    text(nf(sV,1,3), 215, CSizeZ-15); //"is: " + str(is)+", sV: " +
  text("get contours", 35, 84); 
  text("2D-interp.", 35, 109);
  text("invert", 35, 135);
  //text("grid", 35, 160); 
  text("curser: "+str(mouseV),CSizeX-134,CSizeZ-10);
  image(im_xbyw0, CSizeX-156,CSizeZ-43);
    text(nf((mouseX-CSizeX/2)*2*x0Max/CSizeX,1,2),CSizeX-110,CSizeZ-30);
  image(im_zbyzR, CSizeX-82,CSizeZ-44);
  text(nf(-(mouseY-CSizeZ/2)*4*z0Max/CSizeZ,1,2),CSizeX-35,CSizeZ-30);
  text("'p': toggle line profiles",CSizeX-135,20);
  //=======================================================================================
}

float signum(float f) {
  if (f > 0) return 1.0;
  if (f < 0) return -1.0;
  return 0;
}

float HeavySide(float f) {
  if (f > 0) return 1.0;
  if (f < 0) return 0;
  return 0;
}

//Key press events
void keyPressed() {
  if (key == 'i') {
      interpToggle = !interpToggle;
  }
  if (key == 'a') {
      is = abs(is - 1) % Ns; //pressing "+" takes next s
  }
  if (key == 's') {
      is = abs((is + 1) % Ns); //pressing "-" takes previous s
  }
  if (key == 'r') {
      itheta = abs(itheta - 1) % Ntheta; //pressing "+" takes next theta
  }
  if (key == 't') {
      itheta = abs((itheta + 1) % Ntheta); //pressing "-" takes previous theta
  }
  if (key == 'p') {
    LineProfile = (LineProfile + 1) % 3;
  }
  if (key == 'b') {
    BeamToggle = !BeamToggle;
  }
  if (key == 'x') {
    AxisToggle = !AxisToggle;
  }
  if (key == 'g') {
    GridToggle = !GridToggle;
  }
}

//called whenever mouse is moved
void mouseMoved() {
}

// interpolates 2D-array data[i][j] to data[ii][ji]  
// algorithm adapted from http://en.wikipedia.org/wiki/Bilinear_interpolation
float Interp2D(float[][][] data, float ii, float ji) {
  int i1 = floor(ii), i2=i1+1; 
  int j1 = floor(ji), j2=j1+1;
  float datai=0, normd=(i2-i1)*(j2-j1);
  if ((i2<Nx) && (j2<Nz)) {
    datai = (data[i1][j1][ki]/normd) * (i2-ii)*(j2-ji);
    datai = datai + (data[i2][j1][ki]/normd) * (ii-i1)*(j2-ji);
    datai = datai + (data[i1][j2][ki]/normd) * (i2-ii)*(ji-j1);
    datai = datai + (data[i2][j2][ki]/normd) * (ii-i1)*(ji-j1);
  } else {
    datai = data[i1][j1][ki];
  }
  return datai;
}

void UpdateBG() {
  TransmissionIm.loadPixels();
  TransmissionImG.loadPixels();
  
  //Get Maximum of Image for Color normalization
  maxV = 0.0; minV = 0.0; maxRe = 0.0; maxIm = 0.0;
  for (i=0; i<Nx-1; i++) {
    for (j=0; j<Nz-1; j++) {
      if (AxRe[i][j][ki] > maxRe) {maxRe = AxRe[i][j][ki];};
      if (abs(AxIm[i][j][ki]) > maxIm) {maxIm = abs(AxIm[i][j][ki]);};
      if (AlphaIm*AxRe[i][j][ki]+AlphaRe*abs(AxIm[i][j][ki]) > maxV) {maxV = AlphaIm*AxRe[i][j][ki]+AlphaRe*abs(AxIm[i][j][ki]);};
      if (AlphaIm*AxRe[i][j][ki]-AlphaRe*abs(AxIm[i][j][ki]) < minV) {minV = AlphaIm*AxRe[i][j][ki]-AlphaRe*abs(AxIm[i][j][ki]);};
    }
  }
  minV=minV/sigmaInc; 
  maxV=maxV/sigmaInc;
  maxAbsV = max(abs(minV),abs(maxV));
  
  //Set pixel values using a the current data set (comprising a single qudrant only!)
  z0Scale=0.999*Nz/(height/2); 
  w0Scale=0.999*Nx/(width/2);
  for (int x = 0; x < width; x++) {
    for (int y = 0; y < height; y++) {
      int loc = x + y * width;
      r = signum(y-height/2)*AlphaRe*Interp2D(AxIm, abs(x-width/2)*w0Scale, abs(y-height/2)*z0Scale);
      r = r + AlphaIm*Interp2D(AxRe, abs(x-width/2)*w0Scale, abs(y-height/2)*z0Scale);
      r = -(r/sigmaInc)/maxAbsV;
      if (interpToggle==false) {
        TransmissionImG.pixels[loc] = int(128.0*(1.0+r));
        TransmissionIm.pixels[loc] = ColorF(r); //
      } else {
        //show raw data: no Interpolation
        xi = floor(x*(2*Nx-1)/width); yi = floor(y*(2*Nz-1)/height);
        r = signum(yi-(Nz-1))*AlphaRe*AxIm[floor(abs(xi-(Nx-1)))][floor(abs(yi-(Nz-1)))][ki];
        r = r + AlphaIm*AxRe[floor(abs(xi-(Nx-1)))][floor(abs(yi-(Nz-1)))][ki];
        r = -(r/sigmaInc)/maxAbsV;
        TransmissionImG.pixels[loc] = int(128.0*(1.0+r)); //used for Contours
        TransmissionIm.pixels[loc] = ColorF(r); //
      }
    }
  }
  TransmissionIm.updatePixels();
  TransmissionImG.updatePixels();
 
  //get value at mouse position
    r = signum(mouseY-height/2)*AlphaRe*Interp2D(AxIm, abs(mouseX-width/2)*w0Scale, abs(mouseY-height/2)*z0Scale);
    r = r + AlphaIm*Interp2D(AxRe, abs(mouseX-width/2)*w0Scale, abs(mouseY-height/2)*z0Scale);
    mouseV = (r/sigmaInc);
 
  //low-res image for Contours
  if (interpToggle==true) {
    TransmissionImGlr.loadPixels();
    for (int x = 0; x < 2*Nx-1; x++) {
      for (int y = 0; y < 2*Nz-1; y++) {
        int loc = x + y*(2*Nx-1);
        r = signum(y-(Nz-1))*AlphaRe*AxIm[floor(abs(x-(Nx-1)))][floor(abs(y-(Nz-1)))][ki];
        r = r + AlphaIm*AxRe[floor(abs(x-(Nx-1)))][floor(abs(y-(Nz-1)))][ki];
        r = -(r/sigmaInc)/maxAbsV;
        TransmissionImGlr.pixels[loc] = int(128.0*(1.0+r));
      }
    }
    TransmissionImGlr.updatePixels();
  }
}

//taken from http://forum.processing.org/one/topic/creating-contour-maps.html
void drawContours(int i) {
  Blob b;
  EdgeVertex eA,eB;
  float dscaleX=CSizeX/(2*Nx-1);
  float dscaleZ=CSizeZ/(2*Nz-1);
  
  for (int n=0 ; n<theBlobDetection[i].getBlobNb() ; n++) {
    b=theBlobDetection[i].getBlob(n);
    if (b!=null) {
      stroke(i/levels,0,1-i/levels,0.5);      // coloring the contours
      strokeWeight(1);
      for (int m=0;m<b.getEdgeNb();m++) {
        eA = b.getEdgeVertexA(m);
        eB = b.getEdgeVertexB(m);
        if (eA !=null && eB !=null)
          line(
          eA.x*TransmissionImG.width, eA.y*TransmissionImG.height, 
          eB.x*TransmissionImG.width, eB.y*TransmissionImG.height 
            );
      }
    }
  }
}

//taken from http://forum.processing.org/one/topic/creating-contour-maps.html
void GetContours() {
  UpdateBG();
  //Computing Blobs with different thresholds
  if (interpToggle==true) {
    for (int i=0 ; i<levels ; i++) {
      theBlobDetection[i] = new BlobDetection(TransmissionImGlr.width, TransmissionImGlr.height);
      theBlobDetection[i].setThreshold((i/levels)/3); //third because of luminosity?
      theBlobDetection[i].computeBlobs(TransmissionImGlr.pixels);
    }
  } else {
    for (int i=0 ; i<levels ; i++) {
      theBlobDetection[i] = new BlobDetection(TransmissionImG.width, TransmissionImG.height);
      theBlobDetection[i].setThreshold((i/levels)/3); //third because of luminosity?
      theBlobDetection[i].computeBlobs(TransmissionImG.pixels);
    }
  }
}

color ColorF(float r) { //using r in (-1,1), this function returns a corresponding color, i.e. embodies the color-map.
  color ColorV=color(0,0,0,0);
  if (InvertColorToggle==true) {
    ColorV = lerpColor(color(HeavySide(r),0,HeavySide(-r),AlphaColor),color(1,1,1,AlphaColor),1-abs(r)); //red-white-blue
  } else {
    ColorV = color(r*HeavySide(r),0,-r*HeavySide(-r),AlphaColor); //red-black-blue
  }
  return ColorV;
}

